@@ -7,7 +7,7 @@
 #
 #   Copyright (C) 2000-2006  Matt Zimmerman  <mdz@debian.org>
 #   Copyright (C) 2006       Pierre Habouzit <madcoder@debian.org>
-#   Copyright (C) 2016       Robert Luberda  <robert@debian.org>
+#   Copyright (C) 2016-2019  Robert Luberda  <robert@debian.org>
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -98,16 +98,22 @@ def main(config):
 
     all_news = []
     all_changelogs = []
+    all_binnmus = dict()
     notes = []
 
     # Main loop
     for srcpackage, binpackages in source_packages.items():
-        (news, changelogs) = _process_srcpackage(config, seen_db, notes, status,
-                                                 srcpackage, binpackages)
+        (news, changelogs) = _process_srcpackage(config, seen_db, notes,
+                                                 status, srcpackage,
+                                                 binpackages)
         if news:
             all_news.append(news)
         if changelogs:
             all_changelogs.append(changelogs)
+            if changelogs.binnmus:
+                for binnmu in changelogs.binnmus:
+                    all_binnmus.setdefault(binnmu.content, []).append(binnmu)
+
 
         bincount = len(binpackages)
         frontend.update_progress(bincount + int(deb_number/8) - int((deb_number+bincount)/8))
@@ -118,12 +124,22 @@ def main(config):
     seen_db.close_db()
 
     for batch in (all_news, all_changelogs):
-        batch.sort(key=lambda x: (x.urgency, x.package))
+        batch.sort(key=lambda x: (x.numeric_urgency, x.package))
+    for dummy, batch in all_binnmus.items():
+        batch.sort(key=lambda x: (x.header))
+
 
     news    = _join_changes(all_news, source_packages,
                             config.headers, lambda package: _('News for %s') % package)
     changes = _join_changes(all_changelogs, source_packages,
                             config.headers, lambda package: _('Changes for %s') % package)
+    binnmus = _join_binnmus(all_binnmus, config.headers)
+
+    if binnmus:
+        if changes:
+            changes += '\n\n' + binnmus
+        else:
+            changes = binnmus
 
     if config.verbose and notes:
         joined_notes = _("Informational notes") + ":\n\n" + '\n'.join(notes)
@@ -208,16 +224,14 @@ def _process_srcpackage(config, seen_db, notes, status, srcpackage, binpackages)
     if not processpkgs:
         return (None, None)
 
-    (all_news, all_changelogs, all_binnmus) = (None, None, None)
+    (all_news, all_changelogs) = (None, None)
     for pkg in processpkgs:
-        (news, changelog, binnmu) = pkg.extract_changes(config.which, fromversion, config.reverse)
+        (news, changelog) = pkg.extract_changes(config.which, fromversion, config.reverse)
 
         if news and not all_news:
             all_news = news
         if changelog and not all_changelogs:
             all_changelogs = changelog
-        if binnmu and not all_binnmus:
-            all_binnmus = binnmu
 
         if all_changelogs or (all_news and config.which == "news"):
             break
@@ -228,14 +242,6 @@ def _process_srcpackage(config, seen_db, notes, status, srcpackage, binpackages)
             if all_changelogs:
                 break
 
-    # Merge binnmu entries with regular changelog entries.
-    # Assumption: the binnmu version is greater than the last non-binnmu version.
-    if all_binnmus:
-        if all_changelogs:
-            all_changelogs.merge_binnmu(all_binnmus, config.reverse)
-        else:
-            all_changelogs = all_binnmus
-
     if all_news or all_changelogs:
         seen_db[srcpackage] = maxversion
 
@@ -256,6 +262,32 @@ def _join_changes(all_changes, source_packages, show_headers, header_package_get
             changes += '--- %s ---\n%s' % (header, rec.changes)
     return changes
 
+def _join_binnmus(all_binnmus, show_headers):
+    binnmus = ''
+    for content, entries in all_binnmus.items():
+        pkgs = '--- ' + _('Binary NMU of')
+        sep = ': '
+        lastlen = len(pkgs)
+        for entry in entries:
+            hdr = entry.header
+            idx = hdr.find(')')
+            if idx >= 0:
+                hdr = hdr[:idx+1]
+
+            # manually wrap the package lines
+            pkgs += sep
+            lastlen += len(sep)
+            sep = ', '
+            if lastlen + len(hdr) > 75:
+                pkgs += '\n '
+                lastlen = 1;
+            pkgs += hdr
+            lastlen += len(hdr)
+
+        binnmus += pkgs + '\n\n' + content + '\n\n'
+
+    return binnmus
+
 def _display(frontend, changes, title_getter):
     if changes:
         frontend.set_title(title_getter())
