@@ -27,13 +27,13 @@ import sys
 from ALChacks import _
 
 def error(msg):
-    print(_("apt-listchanges: %(msg)s") % {'msg': msg}, file=sys.stderr);
+    print(_("apt-listchanges: %(msg)s") % {'msg': msg}, file=sys.stderr)
 
 def warning(msg):
-    print(_("apt-listchanges warning: %(msg)s") % {'msg': msg}, file=sys.stderr);
+    print(_("apt-listchanges warning: %(msg)s") % {'msg': msg}, file=sys.stderr)
 
 def info(msg):
-    print(_("apt-listchanges: %(msg)s") % {'msg': msg}, file=sys.stdout);
+    print(_("apt-listchanges: %(msg)s") % {'msg': msg}, file=sys.stdout)
 
 def debug(msg):
-    print(msg, file=sys.stdout);
+    print(msg, file=sys.stdout)
