@@ -70,7 +70,11 @@ class gtk_frd(frontend):
                               Gtk.MessageType.QUESTION,
                               Gtk.ButtonsType.YES_NO)
         m.set_default_response(Gtk.ResponseType.YES)
-        m.set_markup("<big><b>%s</b></big>\n\n%s" % (_("Continue Installation?"), _("You can abort the installation if you select 'no'.")))
+        m.set_markup("<big><b>%s</b></big>\n\n%s" % (
+            _("Continue Installation?"),
+            _("Select <i>yes</i> to continue with the installation."
+              "\nSelect <i>no</i> to abort the installation.")
+            ))
         if m.run() == Gtk.ResponseType.NO:
             return False
         return True
